#
import time
import pytest


def test_base(emulator):
    emul, cnx = emulator
    assert emul.params['mode'] == 1
    assert emul.params['addr'] == 5


def check_cmd_get(emulator, cmd, value):
    emul, cnx = emulator
    emul.params[cmd] = value
    cnx.write(b'++%s\r\n' % cmd.encode())
    time.sleep(0.1)
    result = cnx.readlines()
    assert len(result) == 1
    result = result[0].strip()
    if isinstance(value, list):
        value = ' '.join(str(x) for x in value)
    assert result == str(value).encode()


def check_cmd_set(emulator, cmd, value):
    emul, cnx = emulator
    expected = value
    if isinstance(value, list):
        value = ' '.join(str(x) for x in value)
    cnx.write(b'++%s %s\r\n' % (cmd.encode(), str(value).encode()))
    time.sleep(0.1)
    result = cnx.readlines()
    assert len(result) == 0
    assert emul.params[cmd] == expected


def check_cmd_err(emulator, cmd, value):
    emul, cnx = emulator
    defaultvalue = emul.params[cmd]
    cnx.write(b'++%s %s\r\n' % (cmd.encode(), str(value).encode()))
    time.sleep(0.1)
    result = cnx.readlines()
    assert len(result) == 1
    result = result[0].strip()
    assert result == b'Error'
    assert emul.params[cmd] == defaultvalue


test_cmd_get = pytest.mark.parametrize(
    "cmd,value", [
        ('mode', 0), ('mode', 1),
        ('addr', 0), ('addr', 1), ('addr', 30),
        ('addr', [0, 96]), ('addr', [0, 126]), ('addr', [30, 96]),
        ('auto', 0), ('auto', 1),
        ('eoi', 0), ('eoi', 1),
        ('eos', 0), ('eos', 1), ('eos', 3),
        ('eot_enable', 0), ('eot_enable', 1),
        ('eot_char', 0), ('eot_enable', 42), ('eot_char', 255),
        ('read_tmo_ms', 1), ('read_tmo_ms', 3000),
        ('savecfg', 0), ('savecfg', 1),
    ])(check_cmd_get)


test_cmd_set = pytest.mark.parametrize(
    "cmd,value", [
        ('mode', 0), ('mode', 1),
        ('addr', 0), ('addr', 1), ('addr', 30),
        ('addr', [0, 96]), ('addr', [0, 126]), ('addr', [30, 96]),
        ('auto', 0), ('auto', 1),
        ('eoi', 0), ('eoi', 1),
        ('eos', 0), ('eos', 1), ('eos', 3),
        ('eot_enable', 0), ('eot_enable', 1),
        ('eot_char', 0), ('eot_enable', 42), ('eot_char', 255),
        ('read_tmo_ms', 1), ('read_tmo_ms', 3000),
        ('savecfg', 0), ('savecfg', 1),
    ])(check_cmd_set)


test_cmd_err = pytest.mark.parametrize(
    "cmd,value", [
        ('mode', b'2'), ('mode', b'0 0'), ('mode', b'a'),
        ('addr', b'-2'), ('addr', b'31'), ('addr', b'0 0'), ('addr', b'a'),
    ])(check_cmd_err)
